class RequirementError(Exception):
    pass


class BadSignatureError(Exception):
    pass


class SamplerTypeError(Exception):
    pass


class StructuralError(Exception):
    pass


class TokenizeError(Exception):
    pass
